//*****************************************************************************
// File Name            : i2c_sw1.c
//-----------------------------------------------------------------------------
// Function             : OpenCores IP i2c 
//
//-----------------------------------------------------------------------------
// Designer             : yokomizo
//-----------------------------------------------------------------------------
// History
// -.-- 2013/07/17
//*****************************************************************************


#ifdef __hal__
  #include "sys/alt_stdio.h"
  #include "system.h"
  // BASEADDR
  #define PWM0_BASEADDR   0x50000000
  #define PWM1_BASEADDR   0x50000010
  #define I2C_BASEADDR    0xD0000040
  // PRINT macro
  #define M_PRINT(str)  alt_putstr(str);
  #define M_PRINTF(str, ...) printf(str, __VA_ARGS__);
#elif __microblaze__
  #include <stdio.h>
  #include "platform.h"
  #include "xbasic_types.h"
  #include "XIOModule.h"
  // BASEADDR
  #define PWM0_BASEADDR   0xD0000000
  #define PWM1_BASEADDR   0xD0000010
  #define I2C_BASEADDR    0xD0000040
  // PRINT macro
  #define M_PRINT(str)  print(str);
  #define M_PRINTF(str, ...) xil_printf(str, __VA_ARGS__);
#endif

#define GPO      (*(volatile unsigned int *) PIO_0_BASE)
#define PWM0_CTRL (*(volatile unsigned short int *) (PWM0_BASEADDR + 0x0))
#define PWM0_DIV  (*(volatile unsigned short int *) (PWM0_BASEADDR + 0x2))
#define PWM0_PERIOD (*(volatile unsigned short int *) (PWM0_BASEADDR + 0x4))
#define PWM0_DC (*(volatile unsigned short int *) (PWM0_BASEADDR + 0x6))

#define PWM1_CTRL (*(volatile unsigned short int *) (PWM1_BASEADDR + 0x0))
#define PWM1_DIV  (*(volatile unsigned short int *) (PWM1_BASEADDR + 0x2))
#define PWM1_PERIOD (*(volatile unsigned short int *) (PWM1_BASEADDR + 0x4))
#define PWM1_DC (*(volatile unsigned short int *) (PWM1_BASEADDR + 0x6))


#define I2C_PRERL (*(volatile unsigned char *) (I2C_BASEADDR + 0x0))
#define I2C_PRERH (*(volatile unsigned char *) (I2C_BASEADDR + 0x1))
#define I2C_CTR   (*(volatile unsigned char *) (I2C_BASEADDR + 0x2))
#define I2C_TXR   (*(volatile unsigned char *) (I2C_BASEADDR + 0x3))
#define I2C_RXR   (*(volatile unsigned char *) (I2C_BASEADDR + 0x3))
#define I2C_CR    (*(volatile unsigned char *) (I2C_BASEADDR + 0x4))
#define I2C_SR    (*(volatile unsigned char *) (I2C_BASEADDR + 0x4))

void i2c_setup(char per_h_reg,char per_l_reg)
{
	  I2C_PRERL = per_l_reg;
	  I2C_PRERH = per_h_reg;
	  I2C_CTR = 0x80;
	}


void m24fc512_write(unsigned char m_adr,unsigned char wdata)
{
	  //write slv_adr
	  do {
	  I2C_TXR = 0xA2; //slv_adr=1010001,wr=0(write)
	  I2C_CR = 0x90; //start,wr
	  while((0x02 & I2C_SR)==0x02);
	  }	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0x0 ; //meme_adr H
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = m_adr ; //meme_adr L
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = wdata ; //wdata
	  I2C_CR = 0x50; //stop,wr,nack
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  M_PRINTF("wr adr=%x dat=%x\n",m_adr, wdata);
	}

char m24fc512_read(unsigned char m_adr)
{
	  //write slv_adr
	  do {
	  I2C_TXR = 0xA2; //slv_adr=1010001,wr=0(write)
	  I2C_CR = 0x90; //start,wr
	  while((0x02 & I2C_SR)==0x02);
	  }	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0x0 ; //meme_adr H
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = m_adr ; //meme_adr L
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  //read
	  //write slv_adr
	  I2C_TXR = 0xA3; //slv_adr=1010001,wr=1(read)
	  I2C_CR = 0x90; //start,wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0xff; //data=ff
	  I2C_CR = 0x68; //stop,rd,nack
	  while((0x02 & I2C_SR)==0x02);
	  M_PRINTF("rd adr=%x dat=%x\n",m_adr, I2C_RXR);
	  return  I2C_RXR;
	}

void lcd_setcmd(unsigned char wdata)
{
	  //write slv_adr
	  do {
	  I2C_TXR = 0xa0; //slv_adr=1010000,wr=0(write)
	  I2C_CR = 0x90; //start,wr
	  while((0x02 & I2C_SR)==0x02);
	  }	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0x0 ; //Control byte RS=0
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = wdata ; //wdata
	  I2C_CR = 0x50; //stop,wr,nack
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	}

void lcd_setup()
{
      volatile int i;
	  lcd_setcmd(0x01 );
      for(i=0;i<100000;i++);
	  lcd_setcmd(0x38 );
      for(i=0;i<100000;i++);
	  lcd_setcmd(0x0c );
      for(i=0;i<100000;i++);
	  lcd_setcmd(0x06 );
      for(i=0;i<100000;i++);
	  lcd_setcmd(0x80 );
	}

void lcd_write(unsigned char wdata)
{
	  //write slv_adr
	  do {
	  I2C_TXR = 0xa0; //slv_adr=1010000,wr=0(write)
	  I2C_CR = 0x90; //start,wr
	  while((0x02 & I2C_SR)==0x02);
	  }	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0x80 ; //Control byte RS=1
	  I2C_CR = 0x10; //wr
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = wdata ; //wdata
	  I2C_CR = 0x50; //stop,wr,nack
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	}

int tmp102_read()
{
	  int data;
	  //write slv_adr
	  do {
	  I2C_TXR = 0x91; //slv_adr=1001000,wr=1(read)
	  I2C_CR = 0x90; //start,rd
	  while((0x02 & I2C_SR)==0x02);
	  }	  while((0x80 & I2C_SR)==0x80);
	  I2C_TXR = 0xff ; //adr
	  I2C_CR = 0x20; //rd
	  while((0x02 & I2C_SR)==0x02);
	  while((0x80 & I2C_SR)==0x80);
	  //read
	  data = I2C_RXR;
	  I2C_TXR = 0xff; //data=ff
	  I2C_CR = 0x68; //stop,rd,nack
	  while((0x02 & I2C_SR)==0x02);
	  data = data * 256 + (int)I2C_RXR;
	  return data;
	}



int main()
{
  int tmp;
  int tmp1;
  //set up
  M_PRINT("i2c core set up \n");
  i2c_setup(0x01,0x1D);
  M_PRINT("lcd setup\n");
  lcd_setup();
  M_PRINT("-24FC512-\n");
  //24FC512 WRITE
  m24fc512_write( 0x10,0x55);
  //24FC512 READ
  m24fc512_read( 0x10);
  while (1){
	  tmp = tmp102_read();
	  lcd_setcmd(0x80 );
	  tmp1 = (tmp/256);
	  lcd_write( 0x30 + (tmp1/10));
	  tmp1 = tmp1 - (tmp1/10)*10;
	  lcd_write( 0x30 + tmp1);
	  lcd_write( 0x2e);
	  tmp1 = (((0xff&tmp)*100)/256);
	  lcd_write( 0x30 + (tmp1/10));
	  tmp1 = tmp1 - (tmp1/10)*10;
	  lcd_write( 0x30 + tmp1);
  }
  return 0;
}

